from controlpanel import Base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from uwosh.northstar.utils import clone_state, generate_id
from plone.app.workflow.remap import remap_workflow
import validators
from Globals import PersistentMapping
from uwosh.northstar.permissions import managed_permissions

class AddState(Base):
    template = ViewPageTemplateFile('templates/add-new-state.pt')
    
    def __call__(self):
        self.errors = {}

        if not self.request.get('form.actions.add', False):
            return self.handle_response(tmpl=self.template)
        else:
            state = validators.not_empty(self, 'state-name')
            state_id = validators.id(self, 'state-name', self.selected_workflow)

            if not self.errors:
                # must have state to go on
                workflow = self.selected_workflow

                workflow.states.addState(state_id)
                new_state = workflow.states[state_id]
                clone_of_id = self.request.get('clone-from-state')
                if clone_of_id:
                    # manage_copy|paste|clone doesn't work?
                    clone_state(new_state, workflow.states[clone_of_id])

                new_state.title = state

                # if added from transition screen
                referenced_transition = self.request.get('referenced-transition', None)
                if referenced_transition:
                    new_state.transitions = new_state.transitions + (referenced_transition,)

                return self.handle_response(
                    message='"%s" state successfully created.' % new_state.id, 
                    slideto=True,
                    state=new_state
                )
            else:
                return self.handle_response(tmpl=self.template, 
                    justdoerrors=True)

class DeleteState(Base):
    template = ViewPageTemplateFile('templates/delete-state.pt')
    
    def __call__(self):
        self.errors = {}
        state = self.selected_state
        transitions = self.available_transitions
        id = state.id
        
        self.is_using_state = False
        for transition in transitions:
            if transition.new_state_id == id:
                self.is_using_state = True
                break
        
        if self.request.get('form.actions.delete', False) == 'Delete':
            if self.is_using_state:
                replacement = self.request.get('replacement-state', 
                    self.available_states[0].id)
                for transition in self.available_transitions:
                    if id == transition.new_state_id:
                        transition.new_state_id = replacement
                        
                chains = self.portal_workflow.listChainOverrides()
                types_ids = [c[0] for c in chains if self.selected_workflow.id in c[1]]
                remap_workflow(self.context, types_ids, 
                    (self.selected_workflow.id,), {id : replacement})
                
            self.selected_workflow.states.deleteStates([id])
                
            return self.handle_response(message='"%s" state has been successfully deleted.' % id)
        elif self.request.get('form.actions.cancel', False) == 'Cancel':
            return self.handle_response(message='Deleting the "%s" state has been canceled.' % id)
        else:
            return self.handle_response(tmpl=self.template)
       
       
class SaveState(Base):
    
    def update_selected_transitions(self):
        wf = self.selected_workflow
        state = wf.states[self.request.get('selected-state')]
        
        transitions = wf.transitions.objectIds()
        selected_transitions = []
        
        for transition in transitions:
            key = 'transition-%s-state-%s' % (transition, state.id)
            if self.request.has_key(key):
                selected_transitions.append(transition)
                
        state.transitions = tuple(selected_transitions)
        
    def update_state_permissions(self):
        wf = self.selected_workflow
        state = wf.states[self.request.get('selected-state')]
        
        perm_roles = PersistentMapping()
        available_roles = state.getAvailableRoles()
        for managed_perm in managed_permissions:
            selected_roles = []
            
            for role in available_roles:
                key = 'permission-%s-role-%s-state-%s' % (
                    managed_perm['name'], role, state.id)
                if self.request.has_key(key):
                    selected_roles.append(role)
            
            if len(selected_roles) > 0:
                perm_roles[managed_perm['perm']] = tuple(selected_roles)
                
                if managed_perm['perm'] not in wf.permissions:
                    wf.permissions = wf.permissions + (managed_perm['perm'],)
            
            
        state.permission_roles = perm_roles
        
    def update_state_properties(self):
        
        wf = self.selected_workflow
        state = wf.states[self.request.get('selected-state')]
        
        if self.request.has_key('state-%s-initial-state' % state.id):
            wf.initial_state = state.id
            
        title = self.request.get('state-%s-title' % state.id, False)
        if title:
            state.title = title
            
        description = self.request.get('state-%s-description' % state.id, False)
        if description:
            state.description = description
            
    
    def update_state_group_roles(self):
        wf = self.selected_workflow
        state = wf.states[self.request.get('selected-state')]
        
        group_roles = PersistentMapping()
        available_roles = state.getAvailableRoles()
        groups = self.getGroups()
        
        for group in groups:
            selected_roles = []
            
            for role in available_roles:
                key = "group-%s-role-%s-state-%s" % (group['id'], role, state.id)
                if self.request.has_key(key):
                    selected_roles.append(role)
                    
                    
            if len(selected_roles) > 0:
                group_roles[group['id']] = tuple(selected_roles)

                if group['id'] not in wf.groups:
                    wf.groups = wf.groups + (group['id'],)
                    
            
        state.group_roles = group_roles
    
    def __call__(self):
        self.errors = {}
        
        self.update_selected_transitions()
        self.update_state_permissions()
        self.update_state_group_roles()
        self.update_state_properties()
        
        return self.handle_response()